@@ -17,22 +17,21 @@ from powl.objects.obj import (
 def __handle_transition(powl_content: Transition) -> nx.DiGraph:
     # Add artificial start and end nodes
     subgraph = nx.DiGraph()
-    start_node = f"Start_{hash(powl_content)}"
-    end_node = f"End_{hash(powl_content)}"
+    start_node = f"Start_{id(powl_content)}"
+    end_node = f"End_{id(powl_content)}"
     subgraph.add_node(start_node, type="start", visited=True)
     subgraph.add_node(end_node, type="end", visited=True)
-    subgraph.add_node(hash(powl_content), content=powl_content.label, visited=True)
+    subgraph.add_node(id(powl_content), content=powl_content.label, visited=True)
     # Add the edges start -> transition -> end
-    subgraph.add_edge(start_node, hash(powl_content))
-    subgraph.add_edge(hash(powl_content), end_node)
+    subgraph.add_edge(start_node, id(powl_content))
+    subgraph.add_edge(id(powl_content), end_node)
     return subgraph
 
 
 def __handle_silent_transition(powl_content: SilentTransition) -> nx.DiGraph:
-    # That's a silent transition
     subgraph = nx.DiGraph()
-    start_node = f"Start_{hash(powl_content)}"
-    end_node = f"End_{hash(powl_content)}"
+    start_node = f"Start_{id(powl_content)}"
+    end_node = f"End_{id(powl_content)}"
     subgraph.add_node(start_node, type="start", visited=True)
     subgraph.add_node(end_node, type="end", visited=True)
     # Add the edges start -> silent transition -> end
@@ -57,16 +56,16 @@ def __handle_operator_powl(powl_content: OperatorPOWL) -> nx.DiGraph:
         The directed graph representing the OperatorPOWL.
     """
     G = nx.DiGraph()
-    start_event = f"Start_{hash(powl_content)}"
-    end_event = f"End_{hash(powl_content)}"
+    start_event = f"Start_{id(powl_content)}"
+    end_event = f"End_{id(powl_content)}"
     G.add_node(start_event, type="start", visited=True)
     G.add_node(end_event, type="end", visited=True)
     operator = powl_content.operator
 
     if operator == Operator.LOOP and len(powl_content.children) == 2:
         # Add a diverging parallel gateway and a converging parallel gateway
-        diverging_gateway = f"ExclusiveGateway_{hash(powl_content)}_diverging"
-        converging_gateway = f"ExclusiveGateway_{hash(powl_content)}_converging"
+        diverging_gateway = f"ExclusiveGateway_{id(powl_content)}_diverging"
+        converging_gateway = f"ExclusiveGateway_{id(powl_content)}_converging"
         G.add_node(
             diverging_gateway,
             type="diverging",
@@ -82,29 +81,29 @@ def __handle_operator_powl(powl_content: OperatorPOWL) -> nx.DiGraph:
         # Handle the edges now
         do_part = powl_content.children[0]
         redo_part = powl_content.children[1]
-        G.add_node(hash(do_part), content=do_part, visited=False)
-        G.add_node(hash(redo_part), content=redo_part, visited=False)
+        G.add_node(id(do_part), content=do_part, visited=False)
+        G.add_node(id(redo_part), content=redo_part, visited=False)
         G.add_edge(start_event, converging_gateway)
-        G.add_edge(converging_gateway, hash(do_part))
-        G.add_edge(hash(do_part), diverging_gateway)
+        G.add_edge(converging_gateway, id(do_part))
+        G.add_edge(id(do_part), diverging_gateway)
         G.add_edge(diverging_gateway, end_event)
-        G.add_edge(diverging_gateway, hash(redo_part))
-        G.add_edge(hash(redo_part), converging_gateway)
+        G.add_edge(diverging_gateway, id(redo_part))
+        G.add_edge(id(redo_part), converging_gateway)
 
     elif operator == Operator.XOR:
         # One exclusive choice gateway
-        exclusive_gateway_diverging = f"ExclusiveGateway_{hash(powl_content)}_diverging"
+        exclusive_gateway_diverging = f"ExclusiveGateway_{id(powl_content)}_diverging"
         exclusive_gateway_converging = (
-            f"ExclusiveGateway_{hash(powl_content)}_converging"
+            f"ExclusiveGateway_{id(powl_content)}_converging"
         )
         G.add_edge(start_event, exclusive_gateway_diverging)
         G.add_edge(exclusive_gateway_converging, end_event)
 
         for child in powl_content.children:
-            if hash(child) not in G.nodes:
-                G.add_node(hash(child), content=child, visited=False)
-            G.add_edge(exclusive_gateway_diverging, hash(child))
-            G.add_edge(hash(child), exclusive_gateway_converging)
+            if id(child) not in G.nodes:
+                G.add_node(id(child), content=child, visited=False)
+            G.add_edge(exclusive_gateway_diverging, id(child))
+            G.add_edge(id(child), exclusive_gateway_converging)
     else:
         raise ValueError(f"Unsupported operator: {operator}")
     return G
@@ -124,6 +123,7 @@ def __handle_decision_graph(powl_content: DecisionGraph) -> nx.DiGraph:
     G : nx.DiGraph
         The directed graph representing the DecisionGraph.
     """
+
     G = nx.DiGraph()
     edges = __obtain_edges(powl_content, transitive_reduction=False)
     node_edges = {
@@ -136,37 +136,38 @@ def __handle_decision_graph(powl_content: DecisionGraph) -> nx.DiGraph:
     for node in node_edges.keys():
         # Check for end and start
         if type(node) is StartNode:
-            G.add_node(hash(node), type="start", visited=True)
-            # Add an exclusive gateway before it
-            G.add_node(
-                f"ExclusiveGateway_{hash(node)}_afternode",
-                type="exclusive_gateway",
-                visited=True,
-            )
-            # Connect them
-            G.add_edge(hash(node), f"ExclusiveGateway_{hash(node)}_afternode")
-        elif type(node) is EndNode:
-            G.add_node(hash(node), type="end", visited=True)
+            G.add_node(id(node), type="start", visited=True)
             # Add an exclusive gateway after it
             G.add_node(
-                f"ExclusiveGateway_{hash(node)}_beforenode",
+                f"ExclusiveGateway_{id(node)}_afternode",
                 type="exclusive_gateway",
                 visited=True,
             )
             # Connect them
-            G.add_edge(f"ExclusiveGateway_{hash(node)}_beforenode", hash(node))
+            G.add_edge(id(node), f"ExclusiveGateway_{id(node)}_afternode")
+        elif type(node) is EndNode:
+            G.add_node(id(node), type="end", visited=True)
+            # Add an exclusive gateway before it
+            G.add_node(
+                f"ExclusiveGateway_{id(node)}_beforenode",
+                type="exclusive_gateway",
+                visited=True,
+            )
+            # Connect them
+            G.add_edge(f"ExclusiveGateway_{id(node)}_beforenode", id(node))
+        
     G = __add_auxiliary_nodes_before_after(G, node_edges, type="exclusive_gateway")
     for edge in edges:
         src, dst = edge
         src = (
-            f"ExclusiveGateway_{hash(src)}_afternode"
-            if f"ExclusiveGateway_{hash(src)}_afternode" in G.nodes
-            else hash(src)
+            f"ExclusiveGateway_{id(src)}_afternode"
+            if f"ExclusiveGateway_{id(src)}_afternode" in G.nodes
+            else id(src)
         )
         dst = (
-            f"ExclusiveGateway_{hash(dst)}_beforenode"
-            if f"ExclusiveGateway_{hash(dst)}_beforenode" in G.nodes
-            else hash(dst)
+            f"ExclusiveGateway_{id(dst)}_beforenode"
+            if f"ExclusiveGateway_{id(dst)}_beforenode" in G.nodes
+            else id(dst)
         )
         G.add_edge(src, dst)
     return G
@@ -190,30 +191,30 @@ def __add_auxiliary_nodes_before_after(
 ):
     for node in neighbors.keys():
         # add the node
-        if hash(node) in G.nodes:
+        if id(node) in G.nodes:
             # It is handled ad-hoc
             continue
-        G.add_node(hash(node), content=node, visited=False)
+        G.add_node(id(node), content=node, visited=False)
         # Add one exclusive gateway before it and connect it
 
         gateway = (
-            f"ExclusiveGateway_{hash(node)}_beforenode"
+            f"ExclusiveGateway_{id(node)}_beforenode"
             if type == "exclusive_gateway"
-            else f"ParallelGateway_{hash(node)}_beforenode"
+            else f"ParallelGateway_{id(node)}_beforenode"
         )
         if gateway not in G.nodes:
-            G.add_node(gateway, type=type, paired_with=[hash(node)], visited=True)
-        G.add_edge(gateway, hash(node))
+            G.add_node(gateway, type=type, paired_with=[id(node)], visited=True)
+        G.add_edge(gateway, id(node))
 
         # Add one exclusive gateway after it and connect it
         gateway = (
-            f"ExclusiveGateway_{hash(node)}_afternode"
+            f"ExclusiveGateway_{id(node)}_afternode"
             if type == "exclusive_gateway"
-            else f"ParallelGateway_{hash(node)}_afternode"
+            else f"ParallelGateway_{id(node)}_afternode"
         )
         if gateway not in G.nodes:
             G.add_node(gateway, type=type, visited=True)
-        G.add_edge(hash(node), gateway)
+        G.add_edge(id(node), gateway)
     return G
 
 
@@ -233,8 +234,8 @@ def __handle_StrictPartialOrder(powl_content: StrictPartialOrder) -> nx.DiGraph:
     """
     edges = __obtain_edges(powl_content)
     G = nx.DiGraph()
-    start_event = f"Start_{hash(powl_content)}"
-    end_event = f"End_{hash(powl_content)}"
+    start_event = f"Start_{id(powl_content)}"
+    end_event = f"End_{id(powl_content)}"
     G.add_node(start_event, type="start", visited=True)
     G.add_node(end_event, type="end", visited=True)
 
@@ -252,8 +253,8 @@ def __handle_StrictPartialOrder(powl_content: StrictPartialOrder) -> nx.DiGraph:
     end_edges = [node for node, edges in node_edges.items() if not edges["outgoing"]]
 
     # It always has a diverging and converging gateway
-    diverging_gateway = f"ParallelGateway_{hash(powl_content)}_diverging"
-    converging_gateway = f"ParallelGateway_{hash(powl_content)}_converging"
+    diverging_gateway = f"ParallelGateway_{id(powl_content)}_diverging"
+    converging_gateway = f"ParallelGateway_{id(powl_content)}_converging"
     G.add_node(
         diverging_gateway,
         type="diverging",
@@ -273,18 +274,18 @@ def __handle_StrictPartialOrder(powl_content: StrictPartialOrder) -> nx.DiGraph:
 
     # Now, we connect all of the start edges to the diverging gateway
     for start in start_powl:
-        gateway_before_node = f"ParallelGateway_{hash(start)}_beforenode"
+        gateway_before_node = f"ParallelGateway_{id(start)}_beforenode"
         G.add_edge(diverging_gateway, gateway_before_node)
 
     # We connect the end events to the converging gateway
     for end in end_edges:
-        gateway_after_node = f"ParallelGateway_{hash(end)}_afternode"
+        gateway_after_node = f"ParallelGateway_{id(end)}_afternode"
         G.add_edge(gateway_after_node, converging_gateway)
 
     for edge in edges:
         src, dst = edge
-        after_src = f"ParallelGateway_{hash(src)}_afternode"
-        before_dst = f"ParallelGateway_{hash(dst)}_beforenode"
+        after_src = f"ParallelGateway_{id(src)}_afternode"
+        before_dst = f"ParallelGateway_{id(dst)}_beforenode"
         G.add_edge(after_src, before_dst)
     return G
 
@@ -311,6 +312,7 @@ def __generate_submodel(powl_content) -> nx.DiGraph:
         StrictPartialOrder: __handle_StrictPartialOrder,
         DecisionGraph: __handle_decision_graph,
     }
+ 
     if type(powl_content) in handler_map:
         try:
             handler = handler_map[type(powl_content)]
@@ -353,7 +355,7 @@ def __compose_model(
     end_event = end_event_nodes[0]
 
     # Remove the current node from the original graph
-    G.remove_node(hash(current_node))
+    G.remove_node(current_node)
     start_nodes = list(submodel.successors(start_event))
     end_nodes = list(submodel.predecessors(end_event))
     if start_event is None or end_event is None:
@@ -384,7 +386,7 @@ def __compose_model(
         # We have many-to-many connection, or many-to-one, or one-to-many
         # We have to add a parallel gateway in between
         exclusive_gateway_diverging = (
-            f"ExclusiveGateway_{hash(current_node)}_pre_additional"
+            f"ExclusiveGateway_{id(current_node)}_pre_additional"
         )
         # Now, we connect all predecessors to this gateway
         G.add_node(exclusive_gateway_diverging, type="exclusive_gateway", visited=True)
@@ -403,7 +405,7 @@ def __compose_model(
         # We have many-to-many connection, or many-to-one, or one-to-many
         # We have to add a parallel gateway in between
         exclusive_gateway_converging = (
-            f"ExclusiveGateway_{hash(current_node)}_post_additional"
+            f"ExclusiveGateway_{id(current_node)}_post_additional"
         )
         # Now, we connect all successors to this gateway
         G.add_node(exclusive_gateway_converging, type="exclusive_gateway", visited=True)
@@ -529,6 +531,8 @@ def __postprocess_graph(G: nx.DiGraph) -> nx.DiGraph:
         G = G_copy.copy()
     return G_copy
 
+def __print_powl(powl):
+    print(powl)
 
 def apply(powl):
     """
@@ -545,8 +549,8 @@ def apply(powl):
         The converted BPMN model.
     resulting_graph : nx.DiGraph
         The resulting directed graph representing the POWL model.
-    original_element_id_to_hash : dict
-        A mapping from original element IDs to their hashes.
+    original_element_id_to_id : dict
+        A mapping from original element IDs to their ides.
     """
     G = nx.DiGraph()
     # Create the start and end event
@@ -554,17 +558,16 @@ def apply(powl):
     end_event = "EndEvent"
     G.add_node(start_event, type="startEvent", visited=True)
     G.add_node(end_event, type="endEvent", visited=True)
-    G.add_node(hash(powl), content=powl, visited=False)
-    G.add_edge(start_event, hash(powl))
-    G.add_edge(hash(powl), end_event)
+    G.add_node(id(powl), content=powl, visited=False)
+    G.add_edge(start_event, id(powl))
+    G.add_edge(id(powl), end_event)
     resulting_graph = expand_model(powl, G)
     resulting_graph = __postprocess_graph(resulting_graph)
     try:
-        bpmn, original_element_id_to_hash = __transform_to_bpmn(resulting_graph)
+        bpmn, original_element_id_to_id = __transform_to_bpmn(resulting_graph)
     except Exception as e:
         raise ValueError(f"Error transforming graph to BPMN: {e}")
-    return bpmn, resulting_graph, original_element_id_to_hash
-
+    return bpmn, resulting_graph, original_element_id_to_id
 
 def __transform_to_bpmn(G):
     """
@@ -573,7 +576,7 @@ def __transform_to_bpmn(G):
     # create the root
     node_dict = {node: {"incoming": [], "outgoing": []} for node in G.nodes()}
     node_object_mapping = {}
-    original_element_id_to_hash = {}
+    original_element_id_to_id = {}
     for u, v, _ in G.edges(data=True):
         node_dict[u]["outgoing"].append(v)
         node_dict[v]["incoming"].append(u)
@@ -581,38 +584,37 @@ def __transform_to_bpmn(G):
 
     for node, attrs in G.nodes(data=True):
         object = None
-        hashed_id = str(hash(str(node)))
-        print(f"Node: {node}, ID: {hashed_id}, Attributes: {attrs}")
+        ided_id = str(id(str(node)))
         if "Start" in str(node):
-            hashed_id = f"StartEvent_{hashed_id}"
-            object = bpmn.StartEvent(id=hashed_id)
+            ided_id = f"StartEvent_{ided_id}"
+            object = bpmn.StartEvent(id=ided_id)
 
         elif "End" in str(node):
-            hashed_id = f"EndEvent_{hashed_id}"
-            object = bpmn.EndEvent(id=hashed_id)
+            ided_id = f"EndEvent_{ided_id}"
+            object = bpmn.EndEvent(id=ided_id)
 
         elif "Parallel" in str(node):
-            hashed_id = f"ParallelGateway_{hashed_id}"
-            object = bpmn.ParallelGateway(id=hashed_id)
+            ided_id = f"ParallelGateway_{ided_id}"
+            object = bpmn.ParallelGateway(id=ided_id)
 
         elif "Exclusive" in str(node):
-            hashed_id = f"ExclusiveGateway_{hashed_id}"
-            object = bpmn.ExclusiveGateway(id=hashed_id)
+            ided_id = f"ExclusiveGateway_{ided_id}"
+            object = bpmn.ExclusiveGateway(id=ided_id)
 
         elif "Intermediate" in str(node):
 
             if "Catch" in str(node):
-                hashed_id = f"IntermediateCatchEvent_{hashed_id}"
-                object = bpmn.IntermediateCatchEvent(id=hashed_id, name=str(attrs.get("content", "")))
+                ided_id = f"IntermediateCatchEvent_{ided_id}"
+                object = bpmn.IntermediateCatchEvent(id=ided_id, name=str(attrs.get("content", "")))
 
             elif "Throw" in str(node):
-                hashed_id = f"IntermediateThrowEvent_{hashed_id}"
-                object = bpmn.IntermediateThrowEvent(id=hashed_id, name=str(attrs.get("content", "")))
+                ided_id = f"IntermediateThrowEvent_{ided_id}"
+                object = bpmn.IntermediateThrowEvent(id=ided_id, name=str(attrs.get("content", "")))
         else:
             # tasks
-            hashed_id = f"Task_{hashed_id}"
-            object = bpmn.Task(id=hashed_id, name=str(attrs.get("content", "")))
-        original_element_id_to_hash[str(node)] = hashed_id
+            ided_id = f"Task_{ided_id}"
+            object = bpmn.Task(id=ided_id, name=str(attrs.get("content", "")))
+        original_element_id_to_id[str(node)] = ided_id
             
         node_object_mapping[node] = object
         for incoming in node_dict[node]["incoming"]:
@@ -637,4 +639,4 @@ def __transform_to_bpmn(G):
         bpmn.add_node(object)
 
 
-    return bpmn, original_element_id_to_hash    
+    return bpmn, original_element_id_to_id    
