@@ -1,17 +1,28 @@
-from copy import copy
+from copy import deepcopy
 
 from powl.objects.BinaryRelation import BinaryRelation
 from pm4py.objects.process_tree.obj import ProcessTree, Operator
 from typing import List as TList, Optional, Union
 
+from abc import ABC, abstractmethod
 
-class POWL(ProcessTree):
+
+class POWL(ProcessTree, ABC):
+    @abstractmethod
     def simplify_using_frequent_transitions(self) -> "POWL":
         return self
 
+    @abstractmethod
     def simplify(self) -> "POWL":
         return self
 
+    def __str__(self):
+        return self.__repr__()
+
+    @abstractmethod
+    def reduce_silent_transitions(self, add_empty_paths=True) -> "POWL":
+        return self
+
 
 class Transition(POWL):
     transition_id: int = 0
@@ -22,11 +33,20 @@ class Transition(POWL):
         self._identifier = Transition.transition_id
         Transition.transition_id = Transition.transition_id + 1
 
+    def __repr__(self) -> str:
+        return f"Transition(label={self._label}, id={self._identifier})"
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, Transition):
             return self._label == other._label and self._identifier == other._identifier
         return False
 
+    def __copy__(self):
+        return Transition(self._label)
+
+    def __deepcopy__(self, memo):
+        return Transition(self._label)
+
     def equal_content(self, other: object) -> bool:
         if isinstance(other, Transition):
             return self._label == other._label
@@ -46,11 +66,26 @@ class Transition(POWL):
             return True
         return NotImplemented
 
+    def simplify_using_frequent_transitions(self) -> "Transition":
+        return self
+
+    def simplify(self) -> "Transition":
+        return self
+
+    def reduce_silent_transitions(self, add_empty_paths=True) -> "Transition":
+        return self
+
 
 class SilentTransition(Transition):
     def __init__(self) -> None:
         super().__init__(label=None)
 
+    def __copy__(self):
+        return SilentTransition()
+
+    def __deepcopy__(self, memo):
+        return SilentTransition()
+
 
 class FrequentTransition(Transition):
     def __init__(self, label, min_freq: Union[str, int], max_freq: Union[str, int]) -> None:
@@ -66,6 +101,9 @@ class FrequentTransition(Transition):
 
         super().__init__(label=label)
 
+    def __repr__(self):
+        return f"FrequentTransition(activity={self.activity}, min_freq={self.min_freq}, max_freq={self.max_freq})"
+
     def set_skippable(self, skip):
         self.skippable = skip
         self.__update_label()
@@ -94,6 +132,9 @@ class StrictPartialOrder(POWL):
         self._set_order(nodes)
         self.additional_information = None
 
+    def __repr__(self):
+        return f"StrictPartialOrder({self.order.nodes})"
+
     def _set_order(self, nodes: TList[POWL]) -> None:
         self.order = BinaryRelation(nodes)
 
@@ -106,7 +147,6 @@ class StrictPartialOrder(POWL):
     def get_children(self) -> TList[POWL]:
         return self.order.nodes
 
-
     def __lt__(self, other: object) -> bool:
         if isinstance(other, StrictPartialOrder):
             return self.__repr__() < other.__repr__()
@@ -119,7 +159,6 @@ class StrictPartialOrder(POWL):
     partial_order = property(get_order, _set_order)
     children = property(get_children, _set_children)
 
-
     def equal_content(self, other: object) -> bool:
         if not isinstance(other, StrictPartialOrder):
             return False
@@ -152,6 +191,11 @@ class StrictPartialOrder(POWL):
 
         return res
 
+    def reduce_silent_transitions(self, add_empty_paths=True) -> "StrictPartialOrder":
+        new_nodes_map = {node: node.reduce_silent_transitions(add_empty_paths) for node in self.children if
+                         not isinstance(node, SilentTransition)}
+        return self.map_nodes(new_nodes_map)
+
     def simplify(self) -> "StrictPartialOrder":
         simplified_nodes = {}
         sub_nodes = {}
@@ -204,6 +248,17 @@ class StrictPartialOrder(POWL):
                         res.partial_order.add_edge(node_1, node_2)
         return res
 
+    def add_edge(self, source, target):
+        return self.order.add_edge(source, target)
+
+    def map_nodes(self, mapping):
+        res = StrictPartialOrder(list(mapping.values()))
+        for node_1, new_node_1 in mapping.items():
+            for node_2, new_node_2 in mapping.items():
+                if self.partial_order.is_edge(node_1, node_2):
+                    res.partial_order.add_edge(new_node_1, new_node_2)
+        return res
+
 
 class Sequence(StrictPartialOrder):
 
@@ -224,6 +279,11 @@ class Sequence(StrictPartialOrder):
         res = Sequence(new_children)
         return res
 
+    def reduce_silent_transitions(self, add_empty_paths=True) -> "Sequence":
+        new_nodes = [node.reduce_silent_transitions(add_empty_paths) for node in self.children if
+                     not isinstance(node, SilentTransition)]
+        return Sequence(new_nodes)
+
 
 class OperatorPOWL(POWL):
     def __init__(self, operator: Operator, children: TList[POWL]) -> None:
@@ -231,6 +291,9 @@ class OperatorPOWL(POWL):
         self.operator = operator
         self.children = children
 
+    def __repr__(self):
+        return f"{self.operator}({self.children})"
+
     def __lt__(self, other: object) -> bool:
         if isinstance(other, OperatorPOWL):
             return self.__repr__() < other.__repr__()
@@ -258,6 +321,19 @@ class OperatorPOWL(POWL):
                 return False
         return True
 
+    def reduce_silent_transitions(self, add_empty_paths=True) -> POWL:
+
+        new_children = [node.reduce_silent_transitions(add_empty_paths) for node in self.children]
+
+        if self.operator == Operator.XOR:
+            new_children_no_silent = [c for c in new_children if not isinstance(c, SilentTransition)]
+            if len(new_children_no_silent) < len(new_children) - 1:
+                new_children = new_children_no_silent + [SilentTransition()]
+                if len(new_children) == 1:
+                    return new_children[0]
+
+        return OperatorPOWL(operator=self.operator, children=new_children)
+
     def simplify_using_frequent_transitions(self) -> POWL:
         if self.operator is Operator.XOR and len(self.children) == 2:
             child_0 = self.children[0].simplify_using_frequent_transitions()
@@ -361,26 +437,34 @@ class DecisionGraph(POWL):
 
         self.order = order
 
+    def __repr__(self):
+        return f"DecisionGraph({self.children})"
+
     def simplify(self) -> POWL:
         if len(self.children) == 1:
             child_0 = self.children[0]
-            if self.order.is_edge(self.start, self.end):
-                if isinstance(child_0, OperatorPOWL) and child_0.operator is Operator.LOOP:
-                    if isinstance(child_0.children[0], SilentTransition):
-                        return OperatorPOWL(Operator.LOOP, child_0.children).simplify()
-                    elif isinstance(child_0.children[1], SilentTransition):
-                        return OperatorPOWL(Operator.LOOP, list(reversed(child_0.children))).simplify()
-                elif isinstance(child_0, DecisionGraph):
-                    child_0.empty_path = True
-                    child_0.order.add_edge(child_0.start, child_0.end)
-                    return child_0.simplify()
+            skippable = self.order.is_edge(self.start, self.end)
+            repeatable = self.order.is_edge(child_0, child_0)
+
+            if skippable:
+                if repeatable:
+                    return OperatorPOWL(Operator.LOOP, [SilentTransition(), child_0]).simplify()
+                else:
+                    if isinstance(child_0, DecisionGraph):
+                        child_0.empty_path = True
+                        child_0.order.add_edge(child_0.start, child_0.end)
+                        return child_0.simplify()
+                    else:
+                        return OperatorPOWL(Operator.XOR, [SilentTransition(), child_0]).simplify()
+
+            elif repeatable:
+                return OperatorPOWL(Operator.LOOP, [child_0, SilentTransition()]).simplify()
+
             else:
                 return child_0.simplify()
 
-            new_dg = self
-
         else:
-            new_dg = self.__group_pure_xor()
+            new_dg = self
 
             seq = new_dg.__group_start_seq()
             if seq:
@@ -394,18 +478,25 @@ class DecisionGraph(POWL):
             if len(res.children) < len(new_dg.children):
                 return res.simplify()
 
-        new_children_map = {}
-        for child in new_dg.children:
-            s_child = child.simplify()
-            new_children_map[child] = s_child
-        return new_dg.__apply_mapping(new_children_map)
+            new_children_map = {}
+            for child in new_dg.children:
+                s_child = child.simplify()
+                new_children_map[child] = s_child
+            return new_dg.__apply_mapping(new_children_map)
 
     def simplify_using_frequent_transitions(self) -> POWL:
         if len(self.children) == 1:
             child_0 = self.children[0]
+
             if isinstance(child_0, Transition):
-                if self.order.is_edge(self.start, self.end):
-                    return FrequentTransition(label=child_0.label, min_freq=0, max_freq=1)
+                skippable = self.order.is_edge(self.start, self.end)
+                repeatable = self.order.is_edge(child_0, child_0)
+
+                min_freq = 0 if skippable else 1
+                max_freq = "-" if repeatable else 1
+
+                if skippable or repeatable:
+                    return FrequentTransition(label=child_0.label, min_freq=min_freq, max_freq=max_freq)
                 else:
                     return child_0
 
@@ -433,7 +524,6 @@ class DecisionGraph(POWL):
 
         old_size = len(new_dg.children) + 1
         while len(new_dg.children) < old_size:
-            new_dg = new_dg.__group_pure_xor()
             old_size = len(new_dg.children)
             new_dg = new_dg.__group_pure_seq()
 
@@ -498,31 +588,31 @@ class DecisionGraph(POWL):
                 mapping[key] = key
         return mapping
 
-    def __group_pure_xor(self):
-        new_dg = copy(self)
-        visited = set()
-        for child in list(self.children):
-            if child not in visited:
-                visited.add(child)
-                post1 = new_dg.order.get_postset(child)
-                pre1 = new_dg.order.get_preset(child)
-                if len(pre1) > 1 or len(post1) > 1:
-                    candidates_for_xor = [child2 for child2 in new_dg.children if
-                                          pre1 == new_dg.order.get_preset(child2) and post1 == new_dg.order.get_postset(
-                                              child2)]
-                    if len(candidates_for_xor) > 1:
-                        visited.update(candidates_for_xor)
-
-                        new_rel = BinaryRelation(candidates_for_xor)
-                        empty_path = False
-                        edges_to_remove = None
-                        if all(new_dg.order.is_edge(pre, post) for pre in pre1 for post in post1):
-                            empty_path = True
-                            edges_to_remove = [(pre, post) for pre in pre1 for post in post1]
-                        dg_child = DecisionGraph(new_rel, candidates_for_xor, candidates_for_xor, empty_path)
-                        mapping = new_dg.__create_mapping(candidates_for_xor, dg_child)
-                        new_dg = new_dg.__apply_mapping(mapping, edges_to_remove)
-        return new_dg
+    # def __group_pure_xor(self):
+    #     new_dg = deepcopy(self)
+    #     visited = set()
+    #     for child in list(self.children):
+    #         if child not in visited:
+    #             visited.add(child)
+    #             post1 = new_dg.order.get_postset(child)
+    #             pre1 = new_dg.order.get_preset(child)
+    #             if len(pre1) > 1 or len(post1) > 1:
+    #                 candidates_for_xor = [child2 for child2 in new_dg.children if
+    #                                       pre1 == new_dg.order.get_preset(child2) and post1 == new_dg.order.get_postset(
+    #                                           child2)]
+    #                 if len(candidates_for_xor) > 1:
+    #                     visited.update(candidates_for_xor)
+    #
+    #                     new_rel = BinaryRelation(candidates_for_xor)
+    #                     empty_path = False
+    #                     edges_to_remove = None
+    #                     if all(new_dg.order.is_edge(pre, post) for pre in pre1 for post in post1):
+    #                         empty_path = True
+    #                         edges_to_remove = [(pre, post) for pre in pre1 for post in post1]
+    #                     dg_child = DecisionGraph(new_rel, candidates_for_xor, candidates_for_xor, empty_path)
+    #                     mapping = new_dg.__create_mapping(candidates_for_xor, dg_child)
+    #                     new_dg = new_dg.__apply_mapping(mapping, edges_to_remove)
+    #     return new_dg
 
     def __group_pure_seq(self):
         for child in list(self.children):
@@ -540,7 +630,8 @@ class DecisionGraph(POWL):
         start_list = []
         current_dg = self
         while len(current_dg.children) > 1 and len(current_dg.start_nodes) == 1 and not current_dg.order.is_edge(
-                current_dg.start, current_dg.end):
+                current_dg.start, current_dg.end) and current_dg.order.get_preset(current_dg.start_nodes[0]) == {
+            current_dg.start}:
             start = current_dg.start_nodes[0]
             start_list.append(start)
             postset = current_dg.order.get_postset(start)
@@ -563,7 +654,8 @@ class DecisionGraph(POWL):
         end_list = []
         current_dg = self
         while len(current_dg.children) > 1 and len(current_dg.end_nodes) == 1 and not current_dg.order.is_edge(
-                current_dg.start, current_dg.end):
+                current_dg.start, current_dg.end) and current_dg.order.get_postset(current_dg.end_nodes[0]) == {
+            current_dg.end}:
             end = current_dg.end_nodes[0]
             end_list = [end] + end_list
             pretset = current_dg.order.get_preset(end)
@@ -582,6 +674,53 @@ class DecisionGraph(POWL):
             return seq
         return None
 
+    def reduce_silent_transitions(self, add_empty_paths=True) -> "POWL":
+        graph_copy = deepcopy(self)
+
+        mapping = {node: node.reduce_silent_transitions(add_empty_paths) for node in graph_copy.children}
+        order_nodes = [n for n in graph_copy.order.nodes]
+
+        for node, new_node in mapping.items():
+            if isinstance(new_node, SilentTransition):
+                order_nodes.remove(node)
+                for n1 in order_nodes:
+                    for n2 in order_nodes:
+                        if graph_copy.order.is_edge(n1, node) and graph_copy.order.is_edge(node, n2):
+                            graph_copy.order.add_edge(n1, n2)
+
+        mapping = {key: value for key, value in mapping.items() if key in order_nodes}
+
+        if len(mapping.keys()) == 0:
+            return SilentTransition()
+        else:
+            new_start = [value for key, value in mapping.items() if graph_copy.order.is_edge(graph_copy.start, key)]
+            new_end = [value for key, value in mapping.items() if graph_copy.order.is_edge(key, graph_copy.end)]
+            skip = graph_copy.order.is_edge(graph_copy.start, graph_copy.end)
+            new_order = BinaryRelation(list(mapping.values()))
+            for node_1 in mapping.keys():
+                for node_2 in mapping.keys():
+                    if graph_copy.order.is_edge(node_1, node_2):
+                        new_order.add_edge(mapping[node_1], mapping[node_2])
+            if skip and not add_empty_paths:
+                old_skip = self.order.is_edge(self.start, self.end)
+                if not old_skip:
+                    graph_copy.order.remove_edge(graph_copy.start, graph_copy.end)
+                    skip = False
+            return DecisionGraph(new_order, new_start, new_end, skip)
+
+    def map_nodes(self, mapping):
+        assert all(child in mapping.keys() for child in self.children)
+        new_children = list(mapping.values())
+        new_order = BinaryRelation(new_children)
+        for node_1, new_node_1 in mapping.items():
+            for node_2, new_node_2 in mapping.items():
+                if self.order.is_edge(node_1, node_2):
+                    new_order.add_edge(new_node_1, new_node_2)
+        new_start = [mapping[n] for n in self.start_nodes]
+        new_end = [mapping[n] for n in self.end_nodes]
+        empty_path = self.order.is_edge(self.start, self.end)
+        return DecisionGraph(new_order, new_start, new_end, empty_path)
+
 
 class StartNode:
     def __init__(self) -> None:
